=begin

Copyright 2012, Andreas Eisenbarth
All Rights Reserved

Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:         ae_Console.rb
Author:       Andreas Eisenbarth
Description:  
Usage:        menu Window → Ruby Console (WebDialog)
Version:      0.5
Date:         18.06.2012

=end


module AE


class AE::Console < $stdout.class # Sketchup::Console # TODO
  @@dir = File.dirname(__FILE__) # this plugin's folder
  @@open = false
  OSX = ( Object::RUBY_PLATFORM =~ /(darwin)/i ? true : false ) unless defined?(Module.nesting[0]::OSX)
  WINE = ( ((!ENV["WINEPREFIX"].nil? || !ENV["WINELOADER"].nil? || File.exists?("Z:/usr/bin/wine")) && !OSX)? true : false ) unless defined?(Module.nesting[0]::WINE)
  WIN = ( !OSX && !WINE ) unless defined?(Module.nesting[0]::WIN)


  def self.open?
    return @@open # @console_dlg && @console_dlg.visible?
  end


  def initialize
    @console_dlg = nil
    # the context for local variables
    context = Proc.new{}
    @binding = context.binding
    super
  end


	def write(message)
    if @console_dlg && @@open
      if message.to_s[/^\n$/]
        # "puts" sends another separate \n afterwards
        @console_dlg.execute_script( "createNewRow()" ) 
      else
        @console_dlg.execute_script( "result(" + message.to_s.inspect + ", \"ruby\", " + Time.now.to_i.to_s + ")" )
      end
    end
		super(message) # call inherited class write function
	end


  def self.open(instance)
    instance.open
  end


  def open
    # create a webdialog
    @console_dlg = UI::WebDialog.new("Ruby Console (WebDialog)", false, false, 300, 300, width=345, height=400, true)
    @console_dlg.set_size(width, height)
    @console_dlg.set_file( File.join(@@dir, "Console.htm") )
    # callbacks
    @console_dlg.add_action_callback("init"){|@console_dlg, param|
      @console_dlg.set_background_color( @console_dlg.get_default_dialog_color() )
      defaults = load_defaults
      # restore escape characters
      defaults = defaults.gsub(/\&/,'\\').gsub(/\\\\amp;/, '&amp;').gsub(/\\amp;/, '&')
      @console_dlg.execute_script("initialize(#{defaults})")
    }
    # measure the window contents
    @console_dlg.add_action_callback("windowInner") {|@console_dlg, param|
      param = eval(param)
      @windowBorderWidth = (width-param[0].to_i)/2.0 if !@windowBorderWidth
      @windowTitlebarHeight = height-param[1].to_i if !@windowTitlebarHeight
      @console_dlg.execute_script("resizeHeight()")
    }
    # resize the window
    @console_dlg.add_action_callback("resizeHeight") {|@console_dlg, param|
      param = eval(param)
      width = 2*@windowBorderWidth+param[0].to_i
      height = @windowTitlebarHeight+param[1].to_i
      @console_dlg.set_size(width, height)
    }
    # when WebDialog is ready
    @console_dlg.add_action_callback("initialized") { |@console_dlg, param|
      @@open = true
    }
    # for debugging
    @console_dlg.add_action_callback("puts") { |@console_dlg, param|
      puts(param)
    }
    # get command from WebDialog for evaluation and send result back
    @console_dlg.add_action_callback("eval") { |@console_dlg, param|
      command = @console_dlg.get_element_value("returnvalue")
      begin
        result = eval(command, @binding)
        @console_dlg.execute_script( "result(" + result.inspect.inspect + ",\"result\"," + Time.now.to_i.to_s + ")")
      rescue Exception => e  
        @console_dlg.execute_script( "result(" + e.message.inspect + ",\"error\"," + Time.now.to_i.to_s + ")")
        @console_dlg.execute_script( "result(" + e.backtrace.inspect.inspect + ",\"error\"," + Time.now.to_i.to_s + ")")
      end
    }
    # get command from WebDialog for evaluation without return value
    @console_dlg.add_action_callback("do") { |@console_dlg, param|
      command = @console_dlg.get_element_value("returnvalue")
      begin
        eval(command, @binding)
      end
    }
    @console_dlg.set_on_close{
      save_defaults
      @@open = false
    }
    (OSX)? @console_dlg.show_modal : @console_dlg.show
  end


  def load_defaults
    return Sketchup.read_default("Plugins_ae", "Console", "{}")
  end


  def save_defaults
    @console_dlg.execute_script("saveDefaults()")
    defaults = @console_dlg.get_element_value("returnvalue")
    # Replace all escape characters \ by a less used character (ie. &) to avoid multiple \\.
    # Escape all double quotes ".
    defaults = defaults.gsub(/\&/,'&amp;').gsub(/\\/,'&').gsub(/"/,'\"')
    Sketchup.write_default("Plugins_ae", "Console", defaults)
  end



end # class Console



end # module AE



unless file_loaded?(File.basename(__FILE__))
  $old_stdout = $stdout if !$old_stdout # in case you want to turn off traces
  $stdout = AE::Console.new
  $stderr = $stdout # trap error displays as well
  cmd_console = UI::Command.new("Ruby Console (WebDialog)"){
    AE::Console.open($stdout)
  }
  # Menu
  menu_window = UI.menu("Window")
  menu_window.add_item(cmd_console)
end



file_loaded(File.basename(__FILE__))
